/**
 * This file is part of
 * 
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant
 *
 * Copyright (C) 2011 Center for Environmental Systems Research, Kassel, Germany
 * 
 * SesamGIM - Geographical Initialisation for Milieu Agents is free software:
 * You can redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 *  
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Sascha Holzhauer
 * Center for Environmental Systems Research, Kassel
 */
package de.cesr.sonomode.analysis;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections15.Predicate;
import org.apache.log4j.Logger;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import de.cesr.more.basic.MNetworkManager;
import de.cesr.more.measures.network.MNetworkMeasureManager;
import de.cesr.more.measures.network.supply.MCcNetworkMeasureRSupplier;
import de.cesr.more.measures.network.supply.MCentralityNetMSupplier;
import de.cesr.more.measures.network.supply.MNetworkStatisticsSupplier;
import de.cesr.more.measures.node.MNodeMeasureManager;
import de.cesr.more.measures.node.supply.MAvgNearestNeighbourDegree;
import de.cesr.more.measures.util.MScheduleParameters;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.param.SocNetPa;
import de.cesr.sonomode.util.VersionInfo;

/**
 * @author holzhauer
 *
 */
public class SocialNetworkAnalysis {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(SocialNetworkAnalysis.class);
	
	protected static class MilieuPredicate<V> implements Predicate<V> {

		protected int	milieu;

		/**
		 * A new MilieuPredicate that filters the given milieu.
		 * 
		 * @param milieu the milieu to filter
		 */
		protected MilieuPredicate(int milieu) {
			this.milieu = milieu;
		}

		@Override
		public boolean evaluate(Object o) {
			if (!(o instanceof SonomodeAgent)) {
				return false;
			}
			return ((SonomodeAgent) o).getXtremismGp() == milieu;
		}

	}


	@ScheduledMethod(start = 1, interval = ScheduleParameters.END, priority = ScheduleParameters.FIRST_PRIORITY)
	public void analyseNetwork() {

		// <- LOGGING
		logger.info("Analyse network.");
		// LOGGING ->
		
		double analyseTick = (Double) PmParameterManager
				.getParameter(SocNetPa.ANALYSE_NETWORKS_TICK);
		
		if (analyseTick != 0) {
			// set scheduling parameters:
			Map<String, Object> params = new HashMap<String, Object>();
			params.put(MNetworkMeasureManager.ParameterKeys.START.name(),
					new Double(analyseTick > 0 ? analyseTick : 2));

			double interval = analyseTick < 0 ? -analyseTick
					: ScheduleParameters.END;

			params.put(MNetworkMeasureManager.ParameterKeys.INTERVAL.name(),
					new Double(interval));
			params.put(MNetworkMeasureManager.ParameterKeys.END.name(),
					MScheduleParameters.END_TICK);
			params.put(MNetworkMeasureManager.ParameterKeys.PRIORITY.name(),
					MScheduleParameters.FIRST_PRIORITY);
	
			// add measures for overall network:
			 MNetworkMeasureManager.getInstance().addMeasureCalculation(
			 ModelManager.SOCNET_NAME,
			 MNetworkStatisticsSupplier.Short.N_STAT_AVGPATH.getName(),
			 params);
			MNetworkMeasureManager.getInstance().addMeasureCalculation(
					ModelManager.SOCNET_NAME,
					MCcNetworkMeasureRSupplier.Short.R_N_CL_OVERALL.getName(),
					params);
			MNetworkMeasureManager.getInstance().addMeasureCalculation(
					ModelManager.SOCNET_NAME,
					MNetworkStatisticsSupplier.Short.N_STAT_NODES.getName(),
					params);
			MNetworkMeasureManager.getInstance().addMeasureCalculation(
					ModelManager.SOCNET_NAME,
					MNetworkStatisticsSupplier.Short.N_STAT_EDGES.getName(),
					params);
			MNetworkMeasureManager.getInstance().addMeasureCalculation(
					ModelManager.SOCNET_NAME,
					MCentralityNetMSupplier.Short.NET_CEN_OUTDEGREE.getName(),
					params);

			MNodeMeasureManager
					.getInstance()
					.addMeasureCalculation(
							ModelManager.SOCNET_NAME,
							MAvgNearestNeighbourDegree.Short.MC_AVG_NN_DEGREE_UNDIRECTED
									.getName(), params);

			// schedule measure output to DB:
			RunEnvironment
					.getInstance()
					.getCurrentSchedule()
					.schedule(
							ScheduleParameters.createRepeating(
									analyseTick > 0 ? analyseTick : 2,
									analyseTick < 0 ? -analyseTick
											: ScheduleParameters.END, 0),
							this,
							"outputMeasures");
		}
	}
	
	public void outputMeasures() {
		// <- LOGGING
		logger.info("Output network measures");
		// LOGGING ->
		
		MNetworkManager.writeNetworkMeasuresToDb(ModelManager.SOCNET_NAME, " Sonomode: "
				+ VersionInfo.revisionNumber + "/" + VersionInfo.timeStamp, 
				ModelManager.getMan(), (int) RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
	}
}
