/**
 * 
 */
package de.cesr.sonomode.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.collections15.IteratorUtils;
import org.apache.log4j.Logger;

import repast.simphony.context.DefaultContext;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.RandomHelper;
import de.cesr.more.basic.MNetworkManager;
import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.socnet.SonomodeEdge;

/**
 * @author holzhauer
 *
 */
public class SocialContext extends DefaultContext<SonomodeAgent> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(SocialContext.class);
	
	static final double PRECISION = 0.001;
	
	public SocialContext(String name) {
		super(name);
	}

	@ScheduledMethod(start = 1, interval = 1, priority = 100000)
	public void step() {
		SonomodeAgent first;
		SonomodeAgent second;
		
		double firstOpinion;
		double firstUncertainty;
		
		@SuppressWarnings("unchecked")
		MoreNetwork<SonomodeAgent, SonomodeEdge> network = (MoreNetwork<SonomodeAgent, SonomodeEdge>) MNetworkManager.getNetwork(ModelManager.SOCNET_NAME);
		List<SonomodeAgent> remainingAgents = IteratorUtils.toList(this.getAgentLayer(SonomodeAgent.class).iterator());
		
		for (int i = 0; i < ((Integer)PmParameterManager.getParameter(BasicPa.NUM_AGENTS)).intValue(); i++) {
			
			if ((Boolean)PmParameterManager.getParameter(BasicPa.INIT_ALLOW_REDRAWS)) {
				first = this.getRandomObject();
			} else {
				first = remainingAgents.get(RandomHelper.nextIntFromTo(0, remainingAgents.size() - 1));
				remainingAgents.remove(first);
			}
			
			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("Predecessors: " + network.getPredecessors(first));
			}
			// LOGGING ->

			second = network.getRandomPredecessor(first);
			
			// save old opinion- and uncertainty-values of first agent:
			if (second != null) {
				
				firstOpinion = first.getOpinion();
				firstUncertainty = first.getUncertaint();
				
				double change = first.updateProperties(second.getOpinion(), second.getUncertaint(), second) +
						second.updateProperties(firstOpinion, firstUncertainty, first);
				
				if ( change >= PRECISION) {
						// activate edge:
					SonomodeEdge edge = network.getEdge(second, first);
						edge.activate();
						edge.setChange(change);
					}
				
			} else {
				//<- LOGGING
				logger.warn("No partner for agent " + first + " found!");
				// LOGGING ->
			}

		}
		
		// add noise:
		for (SonomodeAgent agent : this.getAgentLayer(SonomodeAgent.class)) {
			// schedule setting of persistent variables in case of synchronous agent triggering
			if (!(Boolean) PmParameterManager.getParameter(BasicPa.ASSYNCHRON_POP_UPDATE)) {
				agent.setPersistentVariables();
			}
			agent.addNoise();
		}
	}
	//@ ScheduledMethod(start=1, interval = 1)
	public double getNumClusters() {
		double epsilon = ((Double)PmParameterManager.getParameter(BasicPa.EPSILON)).doubleValue();

		List<SonomodeAgent> orderedModerateAgents = new ArrayList<SonomodeAgent>();
		
		for (SonomodeAgent agent : this.getAgentLayer(SonomodeAgent.class)) {
			if (agent.isModerate()) {
				orderedModerateAgents.add(agent);
			}
		}
		
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Number of ordered moderates: " + orderedModerateAgents.size());
		}
		if (orderedModerateAgents.size() == 0) {
			logger.warn("No moderates...");
		}
		// LOGGING ->

		Collections.sort(orderedModerateAgents, new Comparator<SonomodeAgent>() {
			@Override
			public int compare(SonomodeAgent o1, SonomodeAgent o2) {
				return Double.compare(o1.getOpinion(), o2.getOpinion());
			}
		});
		
		List<Integer> clusters = new ArrayList<Integer>();
	
		double denominator = 0.0;
		if (orderedModerateAgents.size() > 0) {
			SonomodeAgent first = orderedModerateAgents.get(0);
			// for the first loop, agent = first holds
			int clusterMemberCounter = 0;
			for (SonomodeAgent agent : orderedModerateAgents) {
				if (agent.getOpinion() - first.getOpinion() < epsilon) {
					clusterMemberCounter++;
				} else {
					clusters.add(new Integer(clusterMemberCounter));
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug("Denominator: "
								+ denominator
								+ " || p: "
								+ Math.pow((double) clusterMemberCounter
												/ (double) orderedModerateAgents
														.size(), 2.0));
					}
					// LOGGING ->

					denominator += Math.pow((double) clusterMemberCounter
							/ (double) orderedModerateAgents.size(), 2.0);
					clusterMemberCounter = 1;
				}
				first = agent;
			}
			clusters.add(new Integer(clusterMemberCounter));

			denominator += Math.pow((double) clusterMemberCounter
					/ (double) orderedModerateAgents.size(), 2.0);
		} else {
			denominator = 1.0;
		}
		
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			int counter = 0;
			for (Integer i : clusters) {
				counter += i.intValue();
			}
			logger.debug("Sum of cluster members needs to be equal to number of moderates: "
					+ counter + " = " + orderedModerateAgents.size() + "!");
			logger.debug("n: " + (1.0 / denominator));
		}
		// LOGGING ->

		return 1.0 / denominator;
	}

	/**
	 * Average of the absolute value of the opinions, noted X, which indicates
	 * how extreme is the population."
	 * 
	 * @return average absolute opinion of all (!) agents TODO only moderates?
	 */
	public double getAvgOpinion() {
		double sum = 0.0d;
		for (SonomodeAgent agent : this) {
			sum += Math.abs(agent.getOpinion()); 
		}
		
		if (logger.isDebugEnabled()) {
			logger.debug("The average abs. opinion of " + this.size() + " agents: " + sum / this.size());
		}
		
		//CustomSchedule.inspectSchedule(logger);		
		
		return sum / this.size();
	}
	
	/**
	 * Average of the value of the opinions (not absolute).
	 * @return average opinion of all (!) agents
	 */
	public double getAvgSignedOpinion() {
		double sum = 0.0d;
		for (SonomodeAgent agent : this) {
			sum += agent.getOpinion(); 
		}
		
		if (logger.isDebugEnabled()) {
			logger.debug("The average signed opinion of " + this.size() + " agents: " + sum / this.size());
		}
		
		return sum / this.size();
	}

	/**
	 * Sum of total new links of all agents.
	 * 
	 * @return sum of agent's total new links
	 */
	public int getNumNewLinksTotal() {
		int sum = 0;
		for (SonomodeAgent agent : this) {
			sum += agent.getTotNewLinks();
		}
		return sum;
	}

	/**
	 * Determines the highest convergence tick of all agents in this social
	 * context. NOTE: Calling this method make sense only after termination of
	 * simulation!
	 * 
	 * @return system's convergence tick
	 */
	public float getConverTick() {
		float max = 0;
		for (SonomodeAgent agent : this) {
			max = Math.max(max, agent.getConverTick());
		}
		return max;
	}
}
